@@ -1,7 +1,9 @@
-smahal<-function (z, p, X, caliper, constant=NULL, exact=NULL, nearexact=T, Xextra=NULL, weight=1){
+smahal<-function (z,p,X,caliper,constant=NULL,exact=NULL,nearexact=NULL,Xextra=NULL,weight=NULL){
   if (is.data.frame(X)) X<-as.matrix(X)
   if (is.vector(X)) X<-matrix(X,length(X),1)
   mode(X)<-'numeric'
+  if (is.data.frame(nearexact)) nearexact<-as.matrix(nearexact)
+  if (is.vector(nearexact)) nearexact=as.matrix(nearexact,ncol=1)
   if (is.data.frame(Xextra)) Xextra<-as.matrix(Xextra)
   if (is.vector(Xextra)) Xextra<-matrix(Xextra,ncol=1)
   stopifnot(is.matrix(X))
@@ -9,20 +11,19 @@ smahal<-function (z, p, X, caliper, constant=NULL, exact=NULL, nearexact=T, Xext
   stopifnot(all((z==1)|(z==0)))
   stopifnot(length(z)==(dim(X)[1]))
   stopifnot(caliper>0)
+  if (!is.null(constant)){
+    if (!is.integer(constant)) constant=floor(constant)
+  }
+
   if (!is.null(exact)){
     stopifnot(length(exact)==length(z))
     tb<-table(z,exact)
     if (!all(tb[2,]<=tb[1,])){
-      warning("An exact match for exact is infeasible for every caliper.")
-      if (!nearexact) return(NULL)
+       stop("An exact match for exact is infeasible for every caliper.")
     }
   }
-  if (!is.null(constant)){
-    if (!is.integer(constant)) constant=floor(constant)
-  }
-  if (!is.null(Xextra)){
-    weight=(dim(X)[2])/(dim(Xextra)[2])
-  }
+
+  if (!is.null(Xextra) & is.null(weight)) weight=(dim(matrix(X,nrow=length(z)))[2])/(dim(matrix(Xextra,nrow=length(z)))[2])
 
   n <- dim(X)[1]
 
@@ -33,6 +34,7 @@ smahal<-function (z, p, X, caliper, constant=NULL, exact=NULL, nearexact=T, Xext
     p<-p[o]
     X<-X[o,]
     if (!is.null(exact)) exact<-exact[o]
+    if (!is.null(nearexact)) nearexact<-nearexact[o,]
     if (!is.null(Xextra)) Xextra<-Xextra[o,]
   }
 
@@ -47,6 +49,7 @@ smahal<-function (z, p, X, caliper, constant=NULL, exact=NULL, nearexact=T, Xext
   cv <- diag(rat) %*% cv %*% diag(rat)
   LL <- chol(cv)
   if (!is.null(Xextra)){
+    if (is.vector(Xextra)) Xextra<-matrix(Xextra,ncol=1)
     for (j in 1:(dim(Xextra)[2])) Xextra[, j] <- rank(Xextra[, j])
     cv_ex <- stats::cov(Xextra)
     rat_ex <- sqrt(vuntied/diag(cv_ex))
@@ -62,6 +65,13 @@ smahal<-function (z, p, X, caliper, constant=NULL, exact=NULL, nearexact=T, Xext
     exact0 <- exact[z == 0]
     exact1 <- exact[z == 1]
   }
+  if (!is.null(nearexact)){
+    if (is.vector(nearexact)) nearexact<-matrix(nearexact,ncol=1)
+    nearex0 <- nearexact[z == 0,]
+    nearex1 <- nearexact[z == 1,]
+    if (is.vector(nearex0)) nearex0<-matrix(nearex0,ncol=1)
+    if (is.vector(nearex1)) nearex1<-matrix(nearex1,ncol=1)
+  }
   if (!is.null(Xextra)){
     if (is.vector(Xextra)) Xextra<-matrix(Xextra,ncol=1)
     Xextra0 <- Xextra[z == 0,]
@@ -79,7 +89,7 @@ smahal<-function (z, p, X, caliper, constant=NULL, exact=NULL, nearexact=T, Xext
     d<-abs(p1[i]-p0)
     who<-d<=caliper
     #use exact
-    if ((!is.null(exact)) & (!nearexact)) who <- who & (exact1[i]==exact0)
+    if (!is.null(exact)) who <- who & (exact1[i]==exact0)
     if (!any(who)) return(NULL)
     num <- sum(who)
     #use constant
@@ -102,14 +112,15 @@ smahal<-function (z, p, X, caliper, constant=NULL, exact=NULL, nearexact=T, Xext
       distancei=mvnfast::maha(cc,tt,LL,isChol = TRUE)
     }
 
-    #use nearexact
-    if ((!is.null(exact)) & (nearexact)){
-      nearex=c(nearex, exact1[i]!=exact0[who])
-    }
-
     distance <- c(distance, distancei)
     start_node <- c(start_node, rep(i, num))
     end_node <- c(end_node,as.numeric(row.names(X0)[who]))
+
+    #use nearexact
+    if (!is.null(nearexact)){
+      if (dim(nearexact)[2]==1) nearex=c(nearex, nearex1[i,]!=nearex0[who,])
+      else nearex=rbind(nearex, nearex1[i,]!=nearex0[who,])
+    }
   }
   out <- list(d=distance,start=start_node,end=end_node,nearex=nearex)
   out
